library(pbdMPI, quiet = TRUE)
library(MCMCpack)
init()

start.time <- Sys.time()

.comm.size <- comm.size()
.comm.rank <- comm.rank()
.hostname <- Sys.info()["nodename"]

msg <- sprintf("Process ID %d out of %d processes running on %s\n",
	.comm.rank, .comm.size, .hostname)
comm.cat(msg, all.rank = TRUE, quiet = TRUE)

# Read J and R from the command line, e.g.
# Rscript driver.R 2 1048576
cmd <- commandArgs(TRUE)
J <- as.integer(cmd[1])
R <- as.integer(cmd[2])

alpha <- rep(1,J)
n <- R / .comm.size

# Don't sample them all at once, or we might run out of memory
r <- 0
total <- 0
while (r < n)
{
	X <- rdirichlet( min(1000, n-r), alpha)
	total <- total + sum( apply(X, 1, prod) )
	r <- r + nrow(X)
}

prob.global <- reduce(total / R, op = "sum", rank.dest = 0)

comm.cat("Monte carlo probability:\n", quiet = TRUE)
comm.print(prob.global, digits = 20, quiet = TRUE)

elapsed <- as.numeric(Sys.time() - start.time, units = "secs")
comm.cat("Elapsed secs:", elapsed, "\n", quiet = TRUE)

finalize()
