library(pbdMPI, quiet = TRUE)
init()
.comm.size <- comm.size()
.comm.rank <- comm.rank()

set.seed(1234)

filename <- sprintf("../../data/law82-%d.dat", .comm.rank)
dat <- read.table(filename)

l.n <- nrow(dat)
l.idx <- as.integer(rownames(dat))
n <- allreduce(l.n, op = "sum")
B <- 500
T.boot <- numeric(B)

inner.prod.mpi <- function(x, y, idx)
{
	# idx are global indices
	idx <- idx[idx %in% l.idx]

	# jdx are local indices that index into the local data structures
	jdx <- idx - min(l.idx) + 1

	x.y <- sum(x[jdx] * y[jdx])

	# Just in case this process didn't receive any indices, use x.y = 0
	if (is.na(x.y)) { x.y <- 0 }

	x.y <- allreduce(x.y, op = "sum")
	return(x.y)
}

cor.mpi <- function(x, y, idx)
{
	# Compute the means for the current bootstrap sample given by idx
	# and subtract the means from the data
	ones <- matrix(1/n, nrow = n, ncol = 1)
	m.x <- inner.prod.mpi(x, ones, idx)
	m.y <- inner.prod.mpi(y, ones, idx)
	x <- x - m.x
	y <- y - m.y

	# Compute the correlation
	x.y <- inner.prod.mpi(x, y, idx)
	x.x <- inner.prod.mpi(x, x, idx)
	y.y <- inner.prod.mpi(y, y, idx)

	r <- x.y / ( sqrt(x.x) * sqrt(y.y) )
	return(r)
}

for (b in 1:B)
{
	idx0 <- NULL
	if (.comm.rank == 0)
	{
		idx0 <- as.matrix( sample(1:n, replace = TRUE) )
	}

	idx <- bcast(idx0, rank.source = 0)
	T.boot[b] <- cor.mpi(dat$LSAT, dat$GPA, idx)
}

comm.cat("T.boot:\n", quiet = TRUE)
comm.print(T.boot)

se.boot <- sd(T.boot)
comm.cat("se.boot =", se.boot, "\n", quiet = TRUE)

finalize()
