library(pbdDMAT, quiet=TRUE)
init.grid()
.comm.rank <- comm.rank()
.comm.size <- comm.size()

# Create a np x 1 grid of processes where we'll distribute our matrix
newctxt <- minctxt()
init.grid(NPROW = .comm.size, NPCOL = 1, ICTXT = newctxt)
grid <- blacs(ICTXT = newctxt)
.grid.row <- grid$MYROW
.grid.col <- grid$MYCOL

msg <- sprintf("ID %d: My grid position is (%d,%d)\n",
	.comm.rank, .grid.row, .grid.col)
comm.cat(msg, all.rank = TRUE, quiet = TRUE)

# ------------ Read data for regression ------------
dd <- "../../data"
filename <- sprintf("%s/HondaOdysseyData_part%d.csv", dd, .grid.row)
dat <- read.table(filename, sep = ",", head = TRUE)

msg <- sprintf("ID %d: Read data file %s\n", .comm.rank, filename)
comm.cat(msg, all.rank = TRUE, quiet = TRUE)

# We need a global coding for factor and ordered variables
Model.levels <- unique(unlist(allgather(levels(dat$Model))))
Year.levels <- unique(unlist(allgather(levels(as.ordered(dat$Year)))))

Price <- dat$Price
Model <- factor(dat$Model, levels = Model.levels)
Year <- factor(dat$Year, levels = Year.levels)
Mileage <- dat$Mileage

# Create a model matrix for the local regressors
X.local <- model.matrix(~ Model + Mileage + Year)
y.local <- as.matrix(Price)

# If we don't clear the dimension names, we get an error later
# from redistribute
attributes(X.local)$dimnames <- NULL

# ------------ Create distributed matrices for X and y ------------
k <- ncol(X.local)
l.n <- nrow(X.local)
n <- allreduce(l.n, op = 'sum')
blrows <- allreduce(l.n, op = "max")

X <- new("ddmatrix", Data=X.local, dim=c(n,k), ldim=dim(X.local),
	bldim=c(blrows,k), ICTXT = newctxt)
y <- new("ddmatrix", Data=y.local, dim=c(n,1), ldim=dim(y.local),
	bldim=c(blrows,1), ICTXT = newctxt)
X <- redistribute(X, bldim=c(4,4))
y <- redistribute(y, bldim=c(4,1))

# ------------ Fit regression in parallel ------------
beta.hat <- solve(t(X) %*% X) %*% t(X) %*% y

comm.cat("beta.hat =\n", quiet = TRUE)
global.beta.hat <- as.matrix(beta.hat, proc.dest = 'all')
comm.print(global.beta.hat, quiet = TRUE)

finalize()
