library(pbdDMAT, quiet=TRUE)
init.grid()
.comm.rank <- comm.rank()
.comm.size <- comm.size()

# Create a np x 1 grid of processes where we'll distribute our matrix
newctxt <- minctxt()
init.grid(NPROW = .comm.size, NPCOL = 1, ICTXT = newctxt)
grid <- blacs(ICTXT = newctxt)
.grid.row <- grid$MYROW
.grid.col <- grid$MYCOL

msg <- sprintf("ID %d: My grid position is (%d,%d)\n",
	.comm.rank, grid$MYROW, grid$MYCOL)
comm.cat(msg, all.rank = TRUE, quiet = TRUE)

dd <- "../../../data/"
filename <- sprintf("%s/HondaOdysseyData_part%d.csv", dd, .grid.row)
dat <- read.table(filename, sep = ",", head = TRUE)
X.local <- cbind(dat$Price, dat$Mileage, dat$Year)

msg <- sprintf("ID %d: Read data file %s\n", .comm.rank, filename)
comm.cat(msg, all.rank = TRUE, quiet = TRUE)
comm.print(X.local, all.rank = TRUE)

k <- ncol(X.local)
l.n <- nrow(X.local)
n <- allreduce(l.n, op = 'sum')
blrows <- allreduce(l.n, op = "max")

X <- new("ddmatrix", Data=X.local, dim=c(n,k), ldim=dim(X.local),
	bldim=c(blrows,k), ICTXT = newctxt)
X <- redistribute(X, bldim=c(4,4))

print(X)
X.global <- as.matrix(X)
comm.print(X.global)

finalize()
